@@ -1,9 +1,14 @@
 import os
+from os import walk
 import json
 
 from itertools import chain
 from itertools import count
-from itertools import imap
+
+try:
+    from itertools import imap
+except ImportError:
+    imap = map
 from itertools import repeat
 from itertools import starmap
 
@@ -11,6 +16,7 @@ from functools import partial
 
 from operator import itemgetter
 
+from karld import is_py3
 from karld.iter_utils import i_batch
 from karld.unicode_io import csv_reader
 from karld.unicode_io import get_csv_row_writer
@@ -42,21 +48,37 @@ def ensure_file_path_dir(file_path):
     ensure_dir(os.path.abspath(os.path.dirname(file_path)))
 
 
-def i_read_buffered_file(file_name, buffering=FILE_BUFFER_SIZE):
+def i_read_buffered_file(file_name, buffering=FILE_BUFFER_SIZE, binary=True,
+                         py3_csv_read=False):
     """
     Generator of lines of a file name, with buffering for
     speed.
     """
-    with open(file_name, 'rb', buffering=buffering) as stream:
+    kwargs = dict(buffering=buffering)
+    if is_py3() and py3_csv_read:
+        kwargs.update(dict(newline=''))
+
+    with open(file_name, 'r' + ('b' if binary else 't'), **kwargs) as stream:
         for line in stream:
             yield line
 
 
+i_read_buffered_text_file = partial(i_read_buffered_file, binary=False)
+i_read_buffered_binary_file = partial(i_read_buffered_file, binary=True)
+
+
 def i_get_csv_data(file_name, *args, **kwargs):
     """A generator for reading a csv file.
     """
     buffering = kwargs.get('buffering', FILE_BUFFER_SIZE)
-    for row in csv_reader(i_read_buffered_file(file_name, buffering=buffering), *args, **kwargs):
+    read_file_kwargs = dict(buffering=buffering)
+    if is_py3():
+        read_file_kwargs.update(dict(binary=False))
+        read_file_kwargs.update(dict(py3_csv_read=True))
+
+    data = i_read_buffered_file(file_name, **read_file_kwargs)
+
+    for row in csv_reader(data, *args, **kwargs):
         yield row
 
 
@@ -80,10 +102,18 @@ def write_as_csv(items, file_name, append=False,
     if line_buffer_size is None:
         line_buffer_size = LINE_BUFFER_SIZE
     if append:
-        mode = 'ab'
+        mode = 'a'
     else:
-        mode = 'wtb'
-    with open(file_name, mode, buffering=buffering) as csv_file:
+        mode = 'w'
+
+    kwargs = dict(buffering=buffering)
+    if is_py3():
+        mode += 't'
+        kwargs.update(dict(newline=''))
+    else:
+        mode += 'b'
+
+    with open(file_name, mode, **kwargs) as csv_file:
         write_row = get_csv_row_writer(csv_file)
         batches = i_batch(line_buffer_size, items)
         for batch in batches:
@@ -181,12 +211,17 @@ def split_file_output(name, data, out_dir, max_lines=1100,
     """
     batches = i_batch(max_lines, data)
 
+    if is_py3():
+        join_str = b''
+    else:
+        join_str = ''
+
     index = count()
     for group in batches:
         file_path = os.path.join(out_dir,
                                  "{0}_{1}".format(next(index), name))
-        with open(file_path, 'wt', buffering=buffering) as shard_file:
-            shard_file.write("".join(group))
+        with open(file_path, 'wb', buffering=buffering) as shard_file:
+            shard_file.write(join_str.join(group))
 
 
 def raw_line_reader(file_object):
@@ -195,7 +230,7 @@ def raw_line_reader(file_object):
 
 def split_file(file_path, out_dir=None, max_lines=200000,
                buffering=FILE_BUFFER_SIZE, line_reader=raw_line_reader,
-               split_file_writer=split_file_output):
+               split_file_writer=split_file_output, read_binary=True):
     """
     Opens then shards the file.
 
@@ -219,15 +254,22 @@ def split_file(file_path, out_dir=None, max_lines=200000,
     else:
         ensure_dir(out_dir)
 
-    with open(file_path, 'r', buffering=buffering) as data_file:
-        data = line_reader(data_file)
-        split_file_writer(base_name, data, out_dir, max_lines=max_lines,
-                          buffering=buffering)
+    data_file = i_read_buffered_file(file_path, buffering, binary=read_binary)
+    data = line_reader(data_file)
+    split_file_writer(base_name, data, out_dir, max_lines=max_lines,
+                      buffering=buffering)
 
 
 split_multi_line_csv_file = partial(split_file,
                                     line_reader=csv_reader,
-                                    split_file_writer=split_file_output_csv)
+                                    split_file_writer=split_file_output_csv,
+                                    read_binary=True)
+if is_py3():
+    split_multi_line_csv_file = partial(split_file,
+                                        line_reader=csv_reader,
+                                        split_file_writer=split_file_output_csv,
+                                        read_binary=False)
+
 split_multi_line_csv_file.__doc__ = """
 Split a large csv file without separating newlines in quotes. Runs slower
 than split_file.
@@ -266,6 +308,10 @@ def file_path_and_name(path, base_name):
     return os.path.join(path, base_name), base_name
 
 
+def identity(*args):
+    return args
+
+
 def i_walk_dir_for_paths_names(root_dir):
     """
     Walks a directory yielding the directory of files
@@ -276,9 +322,9 @@ def i_walk_dir_for_paths_names(root_dir):
     """
     return chain.from_iterable(
         (
-            imap(None, repeat(subdir), files)
+            imap(identity, repeat(subdir), files)
             for subdir, files
-            in imap(itemgetter(WALK_SUB_DIR, WALK_FILES), os.walk(root_dir))
+            in imap(itemgetter(WALK_SUB_DIR, WALK_FILES), walk(root_dir))
         )
     )
 
